<?php

namespace Laraautos\Accessory\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litepie\Repository\Filter\RequestFilter;
use Laraautos\Accessory\Forms\Accessory as AccessoryForm;
use Laraautos\Accessory\Http\Requests\AccessoryRequest;
use Laraautos\Accessory\Interfaces\AccessoryRepositoryInterface;
use Laraautos\Accessory\Repositories\Eloquent\Filters\AccessoryResourceFilter;
use Laraautos\Accessory\Repositories\Eloquent\Presenters\AccessoryListPresenter;

/**
 * Resource controller class for accessory.
 */
class AccessoryResourceController extends BaseController
{

    /**
     * Initialize accessory resource controller.
     *
     *
     * @return null
     */
    public function __construct(AccessoryRepositoryInterface $accessory)
    {
        parent::__construct();
        $this->form = AccessoryForm::setAttributes()->toArray();
        $this->modules = $this->modules(config('laraautos.accessory.modules'), 'accessory', guard_url('accessory'));
        $this->repository = $accessory;
    }

    /**
     * Display a list of accessory.
     *
     * @return Response
     */
    public function index(AccessoryRequest $request)
    {

        $pageLimit = $request->input('pageLimit', config('database.pagination.limit'));
        $data = $this->repository
            ->pushFilter(RequestFilter::class)
            ->pushFilter(AccessoryResourceFilter::class)
            ->setPresenter(AccessoryListPresenter::class)
            ->simplePaginate($pageLimit)
            // ->withQueryString()
            ->toArray();

        extract($data);
        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('accessory::accessory.names'))
            ->view('accessory::accessory.index')
            ->data(compact('data', 'meta', 'links', 'modules', 'form'))
            ->output();
    }

    /**
     * Display accessory.
     *
     * @param Request $request
     * @param Model   $accessory
     *
     * @return Response
     */
    public function show(AccessoryRequest $request, AccessoryRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('accessory::accessory.name'))
            ->data(compact('data', 'form', 'modules', 'form'))
            ->view('accessory::accessory.show')
            ->output();
    }

    /**
     * Show the form for creating a new accessory.
     *
     * @param Request $request
     *x
     * @return Response
     */
    public function create(AccessoryRequest $request, AccessoryRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('accessory::accessory.name'))
            ->view('accessory::accessory.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Create new accessory.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(AccessoryRequest $request, AccessoryRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $attributes['user_id'] = user_id();
            $attributes['user_type'] = user_type();
            $repository->create($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.created', ['Module' => trans('accessory::accessory.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('accessory/accessory/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/accessory/accessory'))
                ->redirect();
        }

    }

    /**
     * Show accessory for editing.
     *
     * @param Request $request
     * @param Model   $accessory
     *
     * @return Response
     */
    public function edit(AccessoryRequest $request, AccessoryRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('accessory::accessory.name'))
            ->view('accessory::accessory.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Update the accessory.
     *
     * @param Request $request
     * @param Model   $accessory
     *
     * @return Response
     */
    public function update(AccessoryRequest $request, AccessoryRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $repository->update($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('accessory::accessory.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('accessory/accessory/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('accessory/accessory/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the accessory.
     *
     * @param Model   $accessory
     *
     * @return Response
     */
    public function destroy(AccessoryRequest $request, AccessoryRepositoryInterface $repository)
    {
        try {
            $repository->delete();
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('accessory::accessory.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('accessory/accessory/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('accessory/accessory/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }
}
